/******************
 *
 * opt_IPP.cpp
 * 
 * Description: functions implemented with Intel Performance Primitives
 *
 */

// Copyright 2007 Regents of the University of California

// SETI_BOINC is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2, or (at your option) any later
// version.

// SETI_BOINC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.

// You should have received a copy of the GNU General Public License along
// with SETI_BOINC; see the file COPYING.  If not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


// This file is empty if IPP is not being used for the compile
#ifdef USE_IPP

#include <ipp.h>              

// #####
 /*
#if defined(USE_SSSE3)				  
// #define V8 1						  
#pragma message ("-----SSSE3/tm-----")
#include <ipp_v8.h>					  
#include <tmmintrin.h> // SSSE3       
#include <fvec.h>                     
#elif defined(USE_SSE3)				  
#define T7 1
#pragma message ("-----SSE3/pm-----") 
#include <ipp_t7.h>
#include <pmmintrin.h> // SSE3        
#include <fvec.h>                     
#elif defined(USE_SSE2)
// #define W7 1
#pragma message ("-----SSE2/em-----")
#include <emmintrin.h> //SSE2        
#include <fvec.h>                    
#include <ipp_w7.h>
#elif defined(USE_SSE)
// #define A6 1
#pragma message ("-----SSE/xm-----") 
#include <xmmintrin.h> //SSE         
#include <fvec.h>                    
#include <ipp_a6.h>
#else
#pragma message ("-----MMX-----")
#include <ipp_px.h>
#endif
 */

#include <ippm.h>
#include <ippvm.h>       
#include <ipps.h> 

// #####
#include "optimize.hpp"

//
// PowerSpectrum function.
//
// Author: Joe Segur
//
PwrSpectrumOnly_ptt( ipp_GetPSO )
	{

    ALIGNED_YES( FreqData );
    ALIGNED_YES( PowerSpectrum );

	ippsPowerSpectr_32fc( ( const Ipp32fc* )FreqData, ( Ipp32f* )PowerSpectrum, this_fft_len );
	}

//
// Transpose function.
//
// Author: Joe Segur
//

// x = input row length (FFT length)
// y = output column length (Data points in WU / FFT length)
// r = beginning row number (FFT chunk)

TransposeStrip_ptt( ipp_Transpose )
    {
    int i = sizeof(Ipp32f);

    ALIGNED_YES( in );
    ALIGNED_YES( out );

	ippmTranspose_m_32f( ( const Ipp32f* )in, x*i, i, x, 4, ( Ipp32f* )( out+r ), y*i, i );
	}


//
// Chirp functions.
//
// Author: Joe Segur
//


// First version: generate sincos pairs to 50 bit precision, then reduce and chirp.
// Angle reduction is not needed, but the extra sincos precision is costly.
ChirpData_ptt( ipp_ChirpData_A50 )
	{

    ALIGNED_YES( cx_DataArray );
    ALIGNED_YES( cx_ChirpDataArray );

	if (chirp_rate_ind == 0) {
		ippsCopy_32fc( (const Ipp32fc* )cx_DataArray, ( Ipp32fc* )cx_ChirpDataArray, (int)ul_NumDataPoints );
		return 0;
		}

	Ipp64f srate = chirp_rate * M_PI / (sample_rate * sample_rate);
	int i, chunk = 64;

	Ipp64f* dubBufAng = ippsMalloc_64f( chunk*3 );
	Ipp64f* dubBufSin = &dubBufAng[ chunk ];
	Ipp64f* dubBufCos = &dubBufAng[ chunk*2 ];

	Ipp32f* sngBufSin = ippsMalloc_32f( chunk*2 );
	Ipp32f* sngBufCos = &sngBufSin[ chunk ];

	for ( i = 0; i < ul_NumDataPoints-chunk; i += chunk )
		{
		ippsVectorSlope_64f( dubBufAng, chunk, ( Ipp64f )i, 1.0 );
		ippsSqr_64f_I( dubBufAng, chunk );
		ippsMulC_64f_I( srate, dubBufAng, chunk );
		ippsSinCos_64f_A50( dubBufAng, dubBufSin, dubBufCos, chunk );
		ippsConvert_64f32f( dubBufSin, sngBufSin, chunk );
		ippsConvert_64f32f( dubBufCos, sngBufCos, chunk );
		ippsRealToCplx_32f( sngBufSin, sngBufCos, ( Ipp32fc*) ( cx_ChirpDataArray + i ),chunk);
		ippsMul_32fc_I( (const Ipp32fc* )(cx_DataArray + i ), ( Ipp32fc* )( cx_ChirpDataArray + i ), chunk );
		}

	ippsFree( sngBufSin );
	ippsFree( dubBufAng );
	}

// Second version: generate and reduce double angles, then convert to floats.
// Angle reduction is costly, but the 21 bit sincos faster.
ChirpData_ptt( ipp_ChirpData_A21 )
	{

    ALIGNED_YES( cx_DataArray );
    ALIGNED_YES( cx_ChirpDataArray );

	if (chirp_rate_ind == 0) {
		ippsCopy_32fc( (const Ipp32fc* )cx_DataArray, ( Ipp32fc* )cx_ChirpDataArray, (int)ul_NumDataPoints );
		return 0;
		}

	Ipp64f srate = chirp_rate * 0.5 / (sample_rate * sample_rate);
	int i, chunk = 64;

	Ipp64f* dubBufAng = ippsMalloc_64f( chunk*2 );
	Ipp64f* dubBufInt = &dubBufAng[ chunk ];

	Ipp32f* sngBufAng = ippsMalloc_32f( chunk*3 );
	Ipp32f* sngBufSin = &sngBufAng[ chunk ];
	Ipp32f* sngBufCos = &sngBufAng[ chunk*2 ];

	for ( i = 0; i < ul_NumDataPoints-chunk; i += chunk )
		{
		ippsVectorSlope_64f( dubBufAng, chunk, ( Ipp64f )i, 1.0 );
		ippsSqr_64f_I( dubBufAng, chunk );
		ippsMulC_64f_I( srate, dubBufAng, chunk );
		ippsRound_64f( dubBufAng, dubBufInt, chunk );
		ippsSub_64f_I( dubBufInt, dubBufAng, chunk );
		ippsConvert_64f32f( dubBufAng, sngBufAng, chunk );
		ippsMulC_32f_I( 2.0f*M_PI, sngBufAng, chunk );
		ippsSinCos_32f_A21( sngBufAng, sngBufSin, sngBufCos, chunk );
		ippsRealToCplx_32f( sngBufSin, sngBufCos, ( Ipp32fc*) ( cx_ChirpDataArray + i ),chunk);
		ippsMul_32fc_I( (const Ipp32fc* )(cx_DataArray + i ), ( Ipp32fc* )( cx_ChirpDataArray + i ), chunk );
		}

	ippsFree( sngBufAng );
	ippsFree( dubBufAng );
	}

#endif  // USE_IPP
