/******************
 *
 * optnot.cpp
 * 
 * Description: Routines which must not be optimized.
 *
 * Compiler options should be set so it doesn't try to improve this code.
 *
 */

// Copyright 2007 Regents of the University of California

// SETI_BOINC is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2, or (at your option) any later
// version.

// SETI_BOINC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.

// You should have received a copy of the GNU General Public License along
// with SETI_BOINC; see the file COPYING.  If not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


// =============================================================================
//
// fpu_reduce
//
// The doubles in "angles" are reduced to their fractional parts by removal
// of the integer portion, with result values in the -0.5 to +o.5 range, then
// saved as floats in "v_angle" and "v_angle2".
//
// Designed for x87 fpu in extended precision mode, should work for other fpu
// types if the correct roundVal is used.
//
void fpu_reduce( double* angles, double roundVal, float* v_angle, float* v_angle2 )
	{
	v_angle[0] = angles[0] - ((angles[0] + roundVal) - roundVal);
	v_angle[1] = angles[1] - ((angles[1] + roundVal) - roundVal);
	v_angle[2] = angles[2] - ((angles[2] + roundVal) - roundVal);
	v_angle[3] = angles[3] - ((angles[3] + roundVal) - roundVal);
	v_angle2[0] = angles[4] - ((angles[4] + roundVal) - roundVal);
	v_angle2[1] = angles[5] - ((angles[5] + roundVal) - roundVal);
	v_angle2[2] = angles[6] - ((angles[6] + roundVal) - roundVal);
	v_angle2[3] = angles[7] - ((angles[7] + roundVal) - roundVal);
	}
